Module: RailsStats::Inflector
Overview
The Inflector transforms words from singular to plural, class names to table names, modularized class names to ones without, and class names to foreign keys. The default inflections for pluralization, singularization, and uncountable words are kept in inflections.rb.
The Rails core team has stated patches for the inflections library will not be accepted in order to avoid breaking legacy applications which may be relying on errant inflections. If you discover an incorrect inflection and require it for your application or wish to define rules for languages other than English, please correct or add them yourself (explained below).
Defined Under Namespace
Classes: Inflections
Instance Method Summary collapse
-
#camelize(term, uppercase_first_letter = true) ⇒ Object
By default,
camelizeconverts strings to UpperCamelCase. -
#classify(table_name) ⇒ Object
Create a class name from a plural table name like Rails does for table names to models.
-
#constantize(camel_cased_word) ⇒ Object
Tries to find a constant with the name specified in the argument string.
-
#dasherize(underscored_word) ⇒ Object
Replaces underscores with dashes in the string.
-
#deconstantize(path) ⇒ Object
Removes the rightmost segment from the constant expression in the string.
-
#demodulize(path) ⇒ Object
Removes the module part from the expression in the string.
-
#foreign_key(class_name, separate_class_name_and_id_with_underscore = true) ⇒ Object
Creates a foreign key name from a class name.
-
#humanize(lower_case_and_underscored_word) ⇒ Object
Capitalizes the first word and turns underscores into spaces and strips a trailing "_id", if any.
-
#inflections(locale = :en) ⇒ Object
Yields a singleton instance of Inflector::Inflections so you can specify additional inflector rules.
-
#ordinal(number) ⇒ Object
Returns the suffix that should be added to a number to denote the position in an ordered sequence such as 1st, 2nd, 3rd, 4th.
-
#ordinalize(number) ⇒ Object
Turns a number into an ordinal string used to denote the position in an ordered sequence such as 1st, 2nd, 3rd, 4th.
-
#pluralize(word, locale = :en) ⇒ Object
Returns the plural form of the word in the string.
-
#safe_constantize(camel_cased_word) ⇒ Object
Tries to find a constant with the name specified in the argument string.
-
#singularize(word, locale = :en) ⇒ Object
The reverse of
pluralize, returns the singular form of a word in a string. -
#tableize(class_name) ⇒ Object
Create the name of a table like Rails does for models to table names.
-
#titleize(word) ⇒ Object
Capitalizes all the words and replaces some characters in the string to create a nicer looking title.
-
#underscore(camel_cased_word) ⇒ Object
Makes an underscored, lowercase form from the expression in the string.
Instance Method Details
#camelize(term, uppercase_first_letter = true) ⇒ Object
By default, camelize converts strings to UpperCamelCase. If the argument
to camelize is set to :lower then camelize produces
lowerCamelCase.
camelize will also convert '/' to '::' which is useful for converting
paths to namespaces.
'active_model'.camelize # => "ActiveModel"
'active_model'.camelize(:lower) # => "activeModel"
'active_model/errors'.camelize # => "ActiveModel::Errors"
'active_model/errors'.camelize(:lower) # => "activeModel::Errors"
As a rule of thumb you can think of camelize as the inverse of
underscore, though there are cases where that does not hold:
'SSLError'.underscore.camelize # => "SslError"
270 271 272 273 274 275 276 277 278 |
# File 'lib/rails_stats/inflector.rb', line 270 def camelize(term, uppercase_first_letter = true) string = term.to_s if uppercase_first_letter string = string.sub(/^[a-z\d]*/) { inflections.acronyms[$&] || $&.capitalize } else string = string.sub(/^(?:#{inflections.acronym_regex}(?=\b|[A-Z_])|\w)/) { $&.downcase } end string.gsub(/(?:_|(\/))([a-z\d]*)/i) { "#{$1}#{inflections.acronyms[$2] || $2.capitalize}" }.gsub('/', '::') end |
#classify(table_name) ⇒ Object
Create a class name from a plural table name like Rails does for table
names to models. Note that this returns a string and not a Class (To
convert to an actual class follow classify with constantize).
'egg_and_hams'.classify # => "EggAndHam"
'posts'.classify # => "Post"
Singular names are not handled correctly:
'business'.classify # => "Busines"
352 353 354 355 |
# File 'lib/rails_stats/inflector.rb', line 352 def classify(table_name) # strip out any leading schema name camelize(singularize(table_name.to_s.sub(/.*\./, ''))) end |
#constantize(camel_cased_word) ⇒ Object
Tries to find a constant with the name specified in the argument string.
'Module'.constantize # => Module
'Test::Unit'.constantize # => Test::Unit
The name is assumed to be the one of a top-level constant, no matter whether it starts with "::" or not. No lexical context is taken into account:
C = 'outside'
module M
C = 'inside'
C # => 'inside'
'C'.constantize # => 'outside', same as ::C
end
NameError is raised when the name is not in CamelCase or the constant is unknown.
421 422 423 424 425 426 427 428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 |
# File 'lib/rails_stats/inflector.rb', line 421 def constantize(camel_cased_word) names = camel_cased_word.split('::') names.shift if names.empty? || names.first.empty? names.inject(Object) do |constant, name| if constant == Object constant.const_get(name) else candidate = constant.const_get(name) next candidate if constant.const_defined?(name, false) next candidate unless Object.const_defined?(name) # Go down the ancestors to check it it's owned # directly before we reach Object or the end of ancestors. constant = constant.ancestors.inject do |const, ancestor| break const if ancestor == Object break ancestor if ancestor.const_defined?(name, false) const end # owner is in Object, so raise constant.const_get(name, false) end end end |
#dasherize(underscored_word) ⇒ Object
Replaces underscores with dashes in the string.
'puni_puni'.dasherize # => "puni-puni"
360 361 362 |
# File 'lib/rails_stats/inflector.rb', line 360 def dasherize(underscored_word) underscored_word.tr('_', '-') end |
#deconstantize(path) ⇒ Object
Removes the rightmost segment from the constant expression in the string.
'Net::HTTP'.deconstantize # => "Net"
'::Net::HTTP'.deconstantize # => "::Net"
'String'.deconstantize # => ""
'::String'.deconstantize # => ""
''.deconstantize # => ""
See also demodulize.
388 389 390 |
# File 'lib/rails_stats/inflector.rb', line 388 def deconstantize(path) path.to_s[0...(path.rindex('::') || 0)] # implementation based on the one in facets' Module#spacename end |
#demodulize(path) ⇒ Object
Removes the module part from the expression in the string.
'ActiveRecord::CoreExtensions::String::Inflections'.demodulize # => "Inflections"
'Inflections'.demodulize # => "Inflections"
See also deconstantize.
370 371 372 373 374 375 376 377 |
# File 'lib/rails_stats/inflector.rb', line 370 def demodulize(path) path = path.to_s if i = path.rindex('::') path[(i+2)..-1] else path end end |
#foreign_key(class_name, separate_class_name_and_id_with_underscore = true) ⇒ Object
Creates a foreign key name from a class name.
separate_class_name_and_id_with_underscore sets whether
the method should put '_' between the name and 'id'.
'Message'.foreign_key # => "message_id"
'Message'.foreign_key(false) # => "messageid"
'Admin::Post'.foreign_key # => "post_id"
399 400 401 |
# File 'lib/rails_stats/inflector.rb', line 399 def foreign_key(class_name, separate_class_name_and_id_with_underscore = true) underscore(demodulize(class_name)) + (separate_class_name_and_id_with_underscore ? "_id" : "id") end |
#humanize(lower_case_and_underscored_word) ⇒ Object
Capitalizes the first word and turns underscores into spaces and strips a
trailing "_id", if any. Like titleize, this is meant for creating pretty
output.
'employee_salary'.humanize # => "Employee salary"
'author_id'.humanize # => "Author"
308 309 310 311 312 313 314 315 316 |
# File 'lib/rails_stats/inflector.rb', line 308 def humanize(lower_case_and_underscored_word) result = lower_case_and_underscored_word.to_s.dup inflections.humans.each { |(rule, replacement)| break if result.sub!(rule, replacement) } result.gsub!(/_id$/, "") result.tr!('_', ' ') result.gsub(/([a-z\d]*)/i) { |match| "#{inflections.acronyms[match] || match.downcase}" }.gsub(/^\w/) { $&.upcase } end |
#inflections(locale = :en) ⇒ Object
Yields a singleton instance of Inflector::Inflections so you can specify additional inflector rules. If passed an optional locale, rules for other languages can be specified. If not specified, defaults to :en. Only rules for English are provided.
ActiveSupport::Inflector.inflections(:en) do |inflect|
inflect.uncountable 'rails'
end
210 211 212 213 214 215 216 |
# File 'lib/rails_stats/inflector.rb', line 210 def inflections(locale = :en) if block_given? yield Inflections.instance(locale) else Inflections.instance(locale) end end |
#ordinal(number) ⇒ Object
Returns the suffix that should be added to a number to denote the position in an ordered sequence such as 1st, 2nd, 3rd, 4th.
ordinal(1) # => "st"
ordinal(2) # => "nd"
ordinal(1002) # => "nd"
ordinal(1003) # => "rd"
ordinal(-11) # => "th"
ordinal(-1021) # => "st"
487 488 489 490 491 492 493 494 495 496 497 498 499 500 |
# File 'lib/rails_stats/inflector.rb', line 487 def ordinal(number) abs_number = number.to_i.abs if (11..13).include?(abs_number % 100) "th" else case abs_number % 10 when 1; "st" when 2; "nd" when 3; "rd" else "th" end end end |
#ordinalize(number) ⇒ Object
Turns a number into an ordinal string used to denote the position in an ordered sequence such as 1st, 2nd, 3rd, 4th.
ordinalize(1) # => "1st"
ordinalize(2) # => "2nd"
ordinalize(1002) # => "1002nd"
ordinalize(1003) # => "1003rd"
ordinalize(-11) # => "-11th"
ordinalize(-1021) # => "-1021st"
511 512 513 |
# File 'lib/rails_stats/inflector.rb', line 511 def ordinalize(number) "#{number}#{ordinal(number)}" end |
#pluralize(word, locale = :en) ⇒ Object
Returns the plural form of the word in the string.
If passed an optional locale parameter, the word will be
pluralized using rules defined for that language. By default,
this parameter is set to :en.
'post'.pluralize # => "posts"
'octopus'.pluralize # => "octopi"
'sheep'.pluralize # => "sheep"
'words'.pluralize # => "words"
'CamelOctopus'.pluralize # => "CamelOctopi"
'ley'.pluralize(:es) # => "leyes"
233 234 235 |
# File 'lib/rails_stats/inflector.rb', line 233 def pluralize(word, locale = :en) apply_inflections(word, inflections(locale).plurals) end |
#safe_constantize(camel_cased_word) ⇒ Object
Tries to find a constant with the name specified in the argument string.
'Module'.safe_constantize # => Module
'Test::Unit'.safe_constantize # => Test::Unit
The name is assumed to be the one of a top-level constant, no matter whether it starts with "::" or not. No lexical context is taken into account:
C = 'outside'
module M
C = 'inside'
C # => 'inside'
'C'.safe_constantize # => 'outside', same as ::C
end
nil is returned when the name is not in CamelCase or the constant (or
part of it) is unknown.
'blargle'.safe_constantize # => nil
'UnknownModule'.safe_constantize # => nil
'UnknownModule::Foo::Bar'.safe_constantize # => nil
469 470 471 472 473 474 475 476 |
# File 'lib/rails_stats/inflector.rb', line 469 def safe_constantize(camel_cased_word) constantize(camel_cased_word) rescue NameError => e raise unless e. =~ /(uninitialized constant|wrong constant name) #{const_regexp(camel_cased_word)}$/ || e.name.to_s == camel_cased_word.to_s rescue ArgumentError => e raise unless e. =~ /not missing constant #{const_regexp(camel_cased_word)}\!$/ end |
#singularize(word, locale = :en) ⇒ Object
The reverse of pluralize, returns the singular form of a word in a
string.
If passed an optional locale parameter, the word will be
pluralized using rules defined for that language. By default,
this parameter is set to :en.
'posts'.singularize # => "post"
'octopi'.singularize # => "octopus"
'sheep'.singularize # => "sheep"
'word'.singularize # => "word"
'CamelOctopi'.singularize # => "CamelOctopus"
'leyes'.singularize(:es) # => "ley"
250 251 252 |
# File 'lib/rails_stats/inflector.rb', line 250 def singularize(word, locale = :en) apply_inflections(word, inflections(locale).singulars) end |
#tableize(class_name) ⇒ Object
Create the name of a table like Rails does for models to table names. This
method uses the pluralize method on the last word in the string.
'RawScaledScorer'.tableize # => "raw_scaled_scorers"
'egg_and_ham'.tableize # => "egg_and_hams"
'fancyCategory'.tableize # => "fancy_categories"
338 339 340 |
# File 'lib/rails_stats/inflector.rb', line 338 def tableize(class_name) pluralize(underscore(class_name)) end |
#titleize(word) ⇒ Object
Capitalizes all the words and replaces some characters in the string to
create a nicer looking title. titleize is meant for creating pretty
output. It is not used in the Rails internals.
titleize is also aliased as titlecase.
'man from the boondocks'.titleize # => "Man From The Boondocks"
'x-men: the last stand'.titleize # => "X Men: The Last Stand"
'TheManWithoutAPast'.titleize # => "The Man Without A Past"
'raiders_of_the_lost_ark'.titleize # => "Raiders Of The Lost Ark"
328 329 330 |
# File 'lib/rails_stats/inflector.rb', line 328 def titleize(word) humanize(underscore(word)).gsub(/\b(?<!['’`])[a-z]/) { $&.capitalize } end |
#underscore(camel_cased_word) ⇒ Object
Makes an underscored, lowercase form from the expression in the string.
Changes '::' to '/' to convert namespaces to paths.
'ActiveModel'.underscore # => "active_model"
'ActiveModel::Errors'.underscore # => "active_model/errors"
As a rule of thumb you can think of underscore as the inverse of
camelize, though there are cases where that does not hold:
'SSLError'.underscore.camelize # => "SslError"
291 292 293 294 295 296 297 298 299 300 |
# File 'lib/rails_stats/inflector.rb', line 291 def underscore(camel_cased_word) word = camel_cased_word.to_s.dup word.gsub!('::', '/') word.gsub!(/(?:([A-Za-z\d])|^)(#{inflections.acronym_regex})(?=\b|[^a-z])/) { "#{$1}#{$1 && '_'}#{$2.downcase}" } word.gsub!(/([A-Z\d]+)([A-Z][a-z])/,'\1_\2') word.gsub!(/([a-z\d])([A-Z])/,'\1_\2') word.tr!("-", "_") word.downcase! word end |